package org.gcube.service.idm.liferay;

import java.rmi.ServerException;

import org.gcube.idm.common.is.InfrastrctureServiceClient;
import org.gcube.idm.common.is.IsServerConfig;
import org.gcube.vomanagement.usermanagement.impl.ws.LiferayWSUserManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import jakarta.ws.rs.NotFoundException;

public class LiferayClientFactory {
	private static final Logger logger = LoggerFactory.getLogger(LoggerFactory.class);

	// Service endpoint properties
	private final static String RUNTIME_RESOURCE_NAME = "D4Science Infrastructure Gateway";
	private final static String CATEGORY = "Portal";
	private final static String END_POINT_NAME = "JSONWSUser";
	private final static boolean IS_ROOT_SERVICE = true;

	LiferayWSUserManager client = null;

	private IsServerConfig config;

	public static LiferayWSUserManager createtLiferayClientInstance(IsServerConfig config)
			throws NotFoundException, ServerException {

		String host = config.getServerUrl();
		String schema = config.getProperty("schema");
		String user = config.getProperty("username");
		String password = config.getProperty("password");
		Integer port = Integer.valueOf(config.getProperty("port"));

		LiferayWSUserManager client = null;

		try {
			client = new LiferayWSUserManager(user, password, host, schema, port);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new ServerException("cannot create Liferay client");
		}

		if (client == null) {
			throw new ServerException("cannot create Liferay client");
		}

		logger.info("Liferay object built {} - {}", config.getServerUrl(), config.getName());

		return client;
	}

	public IsServerConfig getConfig() throws NotFoundException, ServerException {
		if (this.config == null) {
			this.config = fetchIsConfig();
		}
		return this.config;
	}

	public LiferayWSUserManager getClient() throws NotFoundException, ServerException {
		if (this.client == null) {
			IsServerConfig configuration = this.getConfig();
			this.client = createtLiferayClientInstance(configuration);
		}
		return this.client;
	}

	private static LiferayClientFactory singleton = new LiferayClientFactory();

	public static LiferayClientFactory getSingleton() {
		if (singleton == null)
			singleton = new LiferayClientFactory();
		return singleton;
	}

	public IsServerConfig fetchIsConfig() throws NotFoundException, ServerException {
		IsServerConfig cfg = InfrastrctureServiceClient.serviceConfigFromIS(RUNTIME_RESOURCE_NAME, CATEGORY,
				END_POINT_NAME,
				IS_ROOT_SERVICE);

		return cfg;
	}

}
